# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from ..types.user_type import UserType
import typing
from ..types.frequency import Frequency
from ..core.request_options import RequestOptions
from ..types.user_accounting_response import UserAccountingResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.growth_accounting_response import GrowthAccountingResponse
from ..types.cohorts_response import CohortsResponse
from ..types.concentration_response import ConcentrationResponse
from ..core.client_wrapper import AsyncClientWrapper


class ProductClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_user_accounting(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        metric: str,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserAccountingResponse]:
        """
        Get the product user accounting for the company associated with the deal.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        metric : str

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserAccountingResponse]
            Successful Response

        Examples
        --------
        from termina import Termina

        client = Termina(
            token="YOUR_TOKEN",
        )
        client.product.get_user_accounting(
            group_id=1,
            deal_id=1,
            user_type="business",
            metric="metric",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/user-accounting",
            method="GET",
            params={
                "user_type": user_type,
                "metric": metric,
                "freq": freq,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[UserAccountingResponse],
                    parse_obj_as(
                        type_=typing.List[UserAccountingResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_growth_accounting(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        metric: str,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[GrowthAccountingResponse]:
        """
        Get the product growth accounting for the company associated with the deal.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        metric : str

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GrowthAccountingResponse]
            Successful Response

        Examples
        --------
        from termina import Termina

        client = Termina(
            token="YOUR_TOKEN",
        )
        client.product.get_growth_accounting(
            group_id=1,
            deal_id=1,
            user_type="business",
            metric="metric",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/growth-accounting",
            method="GET",
            params={
                "user_type": user_type,
                "metric": metric,
                "freq": freq,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[GrowthAccountingResponse],
                    parse_obj_as(
                        type_=typing.List[GrowthAccountingResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_cohorts(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        metric: str,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[CohortsResponse]:
        """
        Get the product cohorts for the company associated with the deal.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        metric : str

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[CohortsResponse]
            Successful Response

        Examples
        --------
        from termina import Termina

        client = Termina(
            token="YOUR_TOKEN",
        )
        client.product.get_cohorts(
            group_id=1,
            deal_id=1,
            user_type="business",
            metric="metric",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/cohorts",
            method="GET",
            params={
                "user_type": user_type,
                "metric": metric,
                "freq": freq,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[CohortsResponse],
                    parse_obj_as(
                        type_=typing.List[CohortsResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_concentration(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        metric: str,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConcentrationResponse:
        """
        Get the product concentration for the company associated with the deal.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        metric : str

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConcentrationResponse
            Successful Response

        Examples
        --------
        from termina import Termina

        client = Termina(
            token="YOUR_TOKEN",
        )
        client.product.get_concentration(
            group_id=1,
            deal_id=1,
            user_type="business",
            metric="metric",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/concentration",
            method="GET",
            params={
                "user_type": user_type,
                "metric": metric,
                "freq": freq,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ConcentrationResponse,
                    parse_obj_as(
                        type_=ConcentrationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncProductClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_user_accounting(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        metric: str,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserAccountingResponse]:
        """
        Get the product user accounting for the company associated with the deal.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        metric : str

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserAccountingResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from termina import AsyncTermina

        client = AsyncTermina(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.product.get_user_accounting(
                group_id=1,
                deal_id=1,
                user_type="business",
                metric="metric",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/user-accounting",
            method="GET",
            params={
                "user_type": user_type,
                "metric": metric,
                "freq": freq,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[UserAccountingResponse],
                    parse_obj_as(
                        type_=typing.List[UserAccountingResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_growth_accounting(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        metric: str,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[GrowthAccountingResponse]:
        """
        Get the product growth accounting for the company associated with the deal.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        metric : str

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GrowthAccountingResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from termina import AsyncTermina

        client = AsyncTermina(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.product.get_growth_accounting(
                group_id=1,
                deal_id=1,
                user_type="business",
                metric="metric",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/growth-accounting",
            method="GET",
            params={
                "user_type": user_type,
                "metric": metric,
                "freq": freq,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[GrowthAccountingResponse],
                    parse_obj_as(
                        type_=typing.List[GrowthAccountingResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_cohorts(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        metric: str,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[CohortsResponse]:
        """
        Get the product cohorts for the company associated with the deal.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        metric : str

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[CohortsResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from termina import AsyncTermina

        client = AsyncTermina(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.product.get_cohorts(
                group_id=1,
                deal_id=1,
                user_type="business",
                metric="metric",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/cohorts",
            method="GET",
            params={
                "user_type": user_type,
                "metric": metric,
                "freq": freq,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[CohortsResponse],
                    parse_obj_as(
                        type_=typing.List[CohortsResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_concentration(
        self,
        group_id: int,
        deal_id: int,
        *,
        user_type: UserType,
        metric: str,
        freq: typing.Optional[Frequency] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConcentrationResponse:
        """
        Get the product concentration for the company associated with the deal.

        Parameters
        ----------
        group_id : int

        deal_id : int

        user_type : UserType

        metric : str

        freq : typing.Optional[Frequency]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConcentrationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from termina import AsyncTermina

        client = AsyncTermina(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.product.get_concentration(
                group_id=1,
                deal_id=1,
                user_type="business",
                metric="metric",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/group/{jsonable_encoder(group_id)}/deal/{jsonable_encoder(deal_id)}/data/concentration",
            method="GET",
            params={
                "user_type": user_type,
                "metric": metric,
                "freq": freq,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ConcentrationResponse,
                    parse_obj_as(
                        type_=ConcentrationResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
